package com.fjhx.redis.redisson;

import com.fjhx.redis.redisson.enums.RedisConnectionType;
import com.fjhx.redis.redisson.properties.RedissonProperties;
import com.fjhx.redis.redisson.strategy.RedissonConfigStrategy;
import com.fjhx.redis.redisson.strategy.impl.ClusterRedissonConfigStrategyImpl;
import com.fjhx.redis.redisson.strategy.impl.MasterslaveRedissonConfigStrategyImpl;
import com.fjhx.redis.redisson.strategy.impl.SentinelRedissonConfigStrategyImpl;
import com.fjhx.redis.redisson.strategy.impl.StandaloneRedissonConfigStrategyImpl;
import lombok.extern.slf4j.Slf4j;
import org.redisson.Redisson;
import org.redisson.config.Config;

/**
 * Redisson配置管理器，用于初始化的redisson实例
 */
@Slf4j
public class RedissonManager {

    private Config config = new Config();

    private Redisson redisson = null;

    public RedissonManager() {

    }

    public RedissonManager(RedissonProperties redissonProperties) {
        //装配开关
        Boolean enabled = redissonProperties.getEnabled();
        if (enabled) {
            try {
                log.debug("RedissonManager初始化完成,当前连接方式:" + redissonProperties.getType() + ",连接地址:" + redissonProperties.getAddress());

                config = RedissonConfigFactory.getInstance().createConfig(redissonProperties);
                redisson = (Redisson) Redisson.create(config);
            } catch (Exception e) {
                log.error("Redisson初始化错误", e);
            }
        }
    }

    public Redisson getRedisson() {
        return redisson;
    }

    /**
     * Redisson连接方式配置工厂
     * 双重检查锁
     */
    static class RedissonConfigFactory {

        private RedissonConfigFactory() {

        }

        private static volatile RedissonConfigFactory factory = null;

        public static RedissonConfigFactory getInstance() {
            if (factory == null) {
                synchronized (Object.class) {
                    if (factory == null) {
                        factory = new RedissonConfigFactory();
                    }
                }
            }
            return factory;
        }

        /**
         * 根据连接类型創建连接方式的配置
         *
         * @param redissonProperties
         * @return Config
         */
        Config createConfig(RedissonProperties redissonProperties) {
            RedisConnectionType connectionType = redissonProperties.getType();
            // 声明连接方式
            RedissonConfigStrategy redissonConfigStrategy;
            if (connectionType.equals(RedisConnectionType.SENTINEL)) {
                //哨兵部署方式
                redissonConfigStrategy = new SentinelRedissonConfigStrategyImpl();
            } else if (connectionType.equals(RedisConnectionType.CLUSTER)) {
                //集群部署方式
                redissonConfigStrategy = new ClusterRedissonConfigStrategyImpl();
            } else if (connectionType.equals(RedisConnectionType.MASTERSLAVE)) {
                //主从部署方式
                redissonConfigStrategy = new MasterslaveRedissonConfigStrategyImpl();
            } else {
                //单机部署
                redissonConfigStrategy = new StandaloneRedissonConfigStrategyImpl();
            }
            Config config = redissonConfigStrategy.createRedissonConfig(redissonProperties);
            return config;
        }
    }

}
