package com.fjhx.rfid.r2000.tcp.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.constant.RedisKeyConstant;
import com.fjhx.entity.rfid.RFIDClient;
import com.fjhx.redis.RedisCache;
import com.fjhx.rfid.r2000.tcp.callback.TcpCallBackSaveLog;
import com.fjhx.rfid.r2000.tcp.service.TcpService;
import com.fjhx.thread.ThreadPoolManager;
import com.fjhx.utils.AjaxResult;
import com.fjhx.web.exception.BaseException;
import com.rfid.uhf.controller.impl.ReaderR2000;
import com.rfid.uhf.service.ReaderR2000Service;
import com.rfid.uhf.service.impl.ReaderR2000ServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;
import java.util.stream.Collectors;

/**
 * @Description:
 * @ClassName: TcpServiceImpl
 * @Author: linqt
 * @Date: 2022/3/17 10:32
 * @Version: 1.0
 */
@Service
public class TcpServiceImpl implements TcpService {

    @Autowired
    private RedisCache redisCache;

    /**
     * 初始化实例集合
     */
    private static Map<String, Map<String, Object>> serverMap = new HashMap<>();

    /**
     * 启动单台
     *
     * @param client
     * @return
     */
    public AjaxResult startOne(RFIDClient client) {
        if (client == null || StringUtils.isBlank(client.getIp())) {
            throw new BaseException("读写器IP地址不能为空！");
        }

        ReaderR2000Service service = null;
        ReaderR2000 reader = null;
        try {
            // redis tcp数据缓存key
            String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, client.getIp());

            service = new ReaderR2000ServiceImpl();
            reader = service.connect(client.getIp(), client.getPort(), new TcpCallBackSaveLog(dataRedisKey, client.getUserName()));

            // 实例静态化
            Map<String, Object> map = new HashMap<>();
            map.put("service", service);
            map.put("reader", reader);
            TcpServiceImpl.serverMap.put(client.getIp(), map);

            // 开始扫描
            boolean result = service.beginInvV2(reader);
            if (!result) {
                // 关闭连接
                service.disconnect(reader);
                throw new BaseException(RFIDConstant.SERVER_START_ERR);
            }
        } catch (Exception e) {
            if (reader != null) {
                // 关闭连接
                service.disconnect(reader);
            }
            e.printStackTrace();
            throw new BaseException(RFIDConstant.SERVER_START_ERR);
        }
        return AjaxResult.success(RFIDConstant.SERVER_START_SUCCESS);
    }

    /**
     * 关闭单台
     *
     * @param client
     * @return
     */
    public AjaxResult closeOne(RFIDClient client) {
        if (client == null || StringUtils.isBlank(client.getIp())) {
            throw new BaseException("读写器IP地址不能为空！");
        }

        Map<String, JSONObject> data = new HashMap<>();
        List<JSONObject> list = new ArrayList<>();
        ReaderR2000Service service;
        ReaderR2000 reader;

        // redis tcp数据缓存key
        String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, client.getIp());
        try {
            // 从静态获取实例
            Map<String, Object> map = TcpServiceImpl.serverMap.get(client.getIp());
            service = (ReaderR2000Service) map.get("service");
            reader = (ReaderR2000) map.get("reader");

            // 结束扫描
            service.stopInvV2(reader);

            // 关闭连接
            service.disconnect(reader);

            // 移除服务
            serverMap.remove(client.getIp());

            // 获取数据缓存
            data = redisCache.getCacheObject(dataRedisKey);
//            System.err.println("当前缓存中取出的数据量:" + data.size());

            // 删除数据缓存
            redisCache.deleteObject(dataRedisKey);

            if (MapUtils.isNotEmpty(data)) {
                list = data.values().stream().collect(Collectors.toList());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new BaseException(RFIDConstant.SERVER_CLOSE_ERR);
        }
        return AjaxResult.success(RFIDConstant.SERVER_CLOSE_SUCCESS, list);
    }

    /**
     * 启动多台
     *
     * @param client
     * @return
     */
    @Override
    public AjaxResult openMore(RFIDClient client) {
        ConcurrentHashMap<String, ConcurrentHashMap<String, Object>> sMap = new ConcurrentHashMap<>();
        ConcurrentHashMap<String, JSONObject> dataMap = new ConcurrentHashMap<>();
        List<JSONObject> list = new ArrayList<>();

        Integer count = 0;
        try {
            if (client == null || CollectionUtils.isEmpty(client.getIpList())) {
                throw new BaseException("读写器IP地址不能为空！");
            }
            CountDownLatch countDownLatch = new CountDownLatch(client.getIpList().size());
            // 启动扫描线程
            for (String ip : client.getIpList()) {
                count++;
                Integer finalCount = count;
                CountDownLatch errCountDownLatch = countDownLatch;
                ThreadPoolManager.execute(() -> {
                    try {
                        // redis tcp数据缓存key
                        String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, ip + "-" + finalCount);
                        ReaderR2000Service service = new ReaderR2000ServiceImpl();
                        ReaderR2000 reader = service.connect(ip, client.getPort(), new TcpCallBackSaveLog(dataRedisKey, null));

                        // 实例存储
                        ConcurrentHashMap<String, Object> map = new ConcurrentHashMap<>();
                        map.put("service", service);
                        map.put("reader", reader);
                        sMap.put(ip + "-" + finalCount, map);
                        // 开始扫描
                        service.beginInvV2(reader);
                    } catch (Exception e) {
                        e.printStackTrace();
                        if (errCountDownLatch != null) {
                            errCountDownLatch.countDown();
                        }
                        throw new BaseException(RFIDConstant.READ_DATA_ERR);
                    }
                });
            }

            // 读取时间
            Thread.sleep(client.getReadTime());

            count = 0;
            // 关闭线程
            for (String ip : client.getIpList()) {
                count++;
                Integer finalCount = count;
                CountDownLatch finalCountDownLatch = countDownLatch;
                try {
                    // redis tcp数据缓存key
                    String dataRedisKey = MessageFormat.format(RedisKeyConstant.RFID_TCP_DATA_REDIS_CACHE_PREFIX, ip + "-" + finalCount);

                    // 获取实例
                    ConcurrentHashMap<String, Object> map = sMap.get(ip + "-" + finalCount);
                    ReaderR2000Service service = (ReaderR2000Service) map.get("service");
                    ReaderR2000 reader = (ReaderR2000) map.get("reader");

                    // 结束扫描
                    service.stopInvV2(reader);

                    // 关闭连接
                    service.disconnect(reader);

                    // 移除服务
                    serverMap.remove(ip + "-" + finalCount);

                    // 获取数据缓存
                    ConcurrentHashMap<String, JSONObject> data = redisCache.getCacheObject(dataRedisKey);
                    if (MapUtils.isEmpty(data)) {
//                        System.err.println(ip + "-" + finalCount + "当前缓存中取出的数据为空！");
                        continue;
                    }
//                    System.err.println(ip + "-" + finalCount + "当前缓存中取出的数据量：" + data.size());
                    dataMap.putAll(data);

                    // 删除数据缓存
                    redisCache.deleteObject(dataRedisKey);
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    if (finalCountDownLatch != null) {
                        finalCountDownLatch.countDown();
                    }
                }
            }
//            System.err.println("最终总结果数量--:" + dataMap.size());

            if (MapUtils.isNotEmpty(dataMap)) {
                list = dataMap.values().stream().collect(Collectors.toList());
            }

            if (countDownLatch != null) {
                countDownLatch.await();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success(RFIDConstant.READ_DATA_SUCCESS, list);
    }
}
